////////////////////////////////////////////////////////////////////////////////
//     Point sprite renders a screen aligned quad for every vertex streamed to
//     the shader, thus minimizing cpu cycles and  drawing calls to the gpu.
//     This technique is mostly used for rendering particles. Sprite size (in pixels)
//     is calculated from Projection Matrix and ViewportHeight(optional).
//     The maximum size is usualy 256 px on newer gpus.
//     This shader is based on Template.fx, all the default comments are deleted
//     so you can easily find how to implement this technique. By Viktor Vicsek 2008
////////////////////////////////////////////////////////////////////////////////

float4x4 tW: WORLD;
float4x4 tV: VIEW;
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state
{
    Texture   = (Tex);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
//float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

/////this will define the size of the quads in pixels if calcPerspective is false
float Size  <string uiname="Sprite Size";>;
/////connect this input to the BackBufferHeight output of the renderer
float ViewportHeight;
/////decide if you want perspective
bool calcPerspective<string uiname="Calculate Scale From Perspective";>;
//yet to be done: allow rotating the TexCoords coz the quads cant be rotated
float rotation<string uiname="Rotate Texcoords";>;
float alpha;
float3 camPos;
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float TexCd1 : TEXCOORD1;
    float TexCd2 : TEXCOORD2;
    float color: COLOR0;
    float Size : PSIZE0;  //////////////////  this allows usage of pointsize
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
    float TexCd1 : TEXCOORD1,
    float color: COLOR0,
    float TexCd2 : TEXCOORD2)
{

    vs2ps Out = (vs2ps)0;

    float dist=distance(camPos,PosO);
    Out.Pos = mul(PosO, tWVP);
    
    ////!!!!!!
    ///Out.Size defines sprite size cant be set later
    if(calcPerspective){
                        Out.Size=TexCd1*Size * tP / Out.Pos.w * ViewportHeight/2;
                        }else{
                        Out.Size=TexCd1*Size;
                        }
    Out.TexCd1 = TexCd;
    Out.color=dist;

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{

    // Compute a 2x2 rotation matrix.
    rotation*= In.TexCd1*6.2832;
    float c = cos(rotation);
    float s = sin(rotation);
    float4 rotationMatrix = float4(c, -s, s, c);
    
    
    In.TexCd-=0.5;
    In.TexCd=mul(In.TexCd,float2x2(rotationMatrix))+0.5;
    float4 col = tex2D(Samp, In.TexCd);
    col.a*=alpha;
    col.rgb*=In.color;
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TPointSprite
{
    pass P0
    {
        /////the next 3 statements are important:
        FillMode = POINT;
        PointScaleEnable = true;
        PointSpriteEnable = true;
        //AlphaBlendEnable = true;
        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_2_0 PS();
        AlphaBlendEnable = false;

        AlphaTestEnable = true;
        AlphaFunc = Greater;
        AlphaRef = 245;

        ZEnable = true;
        ZWriteEnable = true;

        CullMode = None;
    }
    pass P1
    {
        /////the next 3 statements are important:
        FillMode = POINT;
        PointScaleEnable = true;
        PointSpriteEnable = true;
        //AlphaBlendEnable = true;
        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_2_0 PS();
        AlphaBlendEnable = true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;

        AlphaTestEnable = true;
        AlphaFunc = LessEqual;
        AlphaRef = 245;

        ZEnable = true;
        ZWriteEnable = false;

        CullMode = None;
    }

}
